@@ -455,7 +455,14 @@ break your code.
 
 You can enable this setting temporarily by setting the HYPOTHESIS_STRICT_MODE
 environment variable to the string 'true'.
-"""
+""",
+    deprecation_message="""
+Strict mode is deprecated and will go away in a future version of Hypothesis.
+
+To get the same behaviour, use
+warnings.simplefilter('error', HypothesisDeprecationWarning).
+""",
+    future_default=False,
 )
 
 settings.define_setting(
@@ -615,6 +622,21 @@ attempting to actually execute your test.
 """
 )
 
+settings.define_setting(
+    'deadline',
+    default=not_set,
+    description=u"""
+If set, a time in milliseconds (which may be a float to express
+smaller units of time) that a test is not allowed to exceed. Tests which take
+longer than that will be converted into errors. Set this to None to disable
+this behaviour entirely.
+
+In future this will default to 200. For now, a
+HypothesisDeprecationWarning will be emitted if you exceed that default
+deadline and have not explicitly set a deadline yourself.
+"""
+)
+
 settings.lock_further_definitions()
 
 settings.register_profile('default', settings())
@@ -623,18 +645,10 @@ assert settings.default is not None
 
 
 def note_deprecation(message, s=None):
-    # If *either* self or the current default are non-strict
-    # then this should be an error. This is to handle e.g. the case
-    # where defining a new setting while non-strict updates a
-    # profile which is strict. This should not be an error, but
-    # using the profile here would cause it to be one.
     if s is None:
         s = settings.default
     assert s is not None
-    strict = settings.default.strict and s.strict
     verbosity = s.verbosity
     warning = HypothesisDeprecationWarning(message)
-    if strict:
-        raise warning
-    elif verbosity > Verbosity.quiet:
+    if verbosity > Verbosity.quiet:
         warnings.warn(warning, stacklevel=3)
